/*
Picsy v1.07   PIC and AVR programmer
Copyright (C) 1996-2000  Johan Dahlberg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


  Project: PICSY
  File:    Devices.h
  Version: 1.07
  Created: 1996-11-01
  Changed: 2000-01-28
  Author:  Johan Dahlberg, johan_d@lysator.liu.se

  PIC command definitions and data buffers
*/

#include "types.h"

#ifndef DEVICES_H
#define DEVICES_H 1

/* PIC programming commands */
#define PCMD_LOADCFG	0x00	/* Load configuration */
#define PCMD_LOADPGM	0x02	/* Load data for program memory */
#define PCMD_READPGM	0x04	/* Read data from program memory */
#define PCMD_INCADDR	0x06	/* Increment address counter */
#define PCMD_PROGRAM	0x08	/* Begin programming */
#define PCMD_LOADDATA	0x03	/* Load data for data memory */
#define PCMD_READDATA	0x05	/* Read data from data memory */
#define PCMD_ERASE1     0x01    /* Bulk erase command 1 */
#define PCMD_ERASE2     0x07    /* Bulk erase command 2 */

/* Data buffer */
#define BUFFER_SIZE	0x20000
#define BUF_EMPTYEVEN	0xff	/* Not-programmed byte-state */
#define BUF_EMPTYODD	0x3f
extern UBYTE buffer[BUFFER_SIZE];	/* Buffer for program data */
extern UBYTE buf_flag[BUFFER_SIZE/16];  /* buf_flag[i] is set if any byte in
					   buffer[i .. i+15] is set on hex file read */


/* Area definitions for program(), verify() and read() in Program.c */
#define AREA_PROGRAM	0x01	/* Program/verify program memory */
#define AREA_DATA	0x02	/* Program/verify data memory */
#define AREA_CONFIG	0x04	/* Program/verify configuration word */
#define AREA_ID         0x08    /* ID bits */



/* AVR programming commands */
#define ACMD_PGMEN      0xac530000 /* Enable serial programming */
#define ACMD_ERASE      0xac800000 /* Chip erase */
#define ACMD_RDPMEM     0x20000000 /* Read program memory */
#define ACMD_WRPMEM     0x40000000 /* Write program memory */
#define ACMD_RDEMEM     0xa0000000 /* Read EEPROM memory */
#define ACMD_WREMEM     0xc0000000 /* Write EEPROM memory */
#define ACMD_WRLOCK     0xacff0000 /* Write lock bits */
#define ACMD_WRFUSE     0xacbf0000 /* Write fuse bits (device dependent command) */
#define ACMD_RDDEV      0x30000000 /* Read device code */


/* AVR flag groups */
#define AGRP_DEVID       1          /* Device ID code group */
#define AGRP_LOCK        2          /* Lock bits supported by device */
#define AGRP_FUSE        3          /* Fuse bits supported by device */

/* AVR device signature codes */
/*
#define DEV_AT90S1200 0x1e9001
#define DEV_AT90S2313 0x1e9101
#define DEV_AT90S2323 0x1e9102
#define DEV_AT90S2333 0x1e9105
#define DEV_AT90S2343 0x1e9103
#define DEV_AT90S4414 0x1e9201
#define DEV_AT90S4433 0x1e9202
#define DEV_AT90S4434 0x1e9203
#define DEV_AT90S8515 0x1e9301
#define DEV_AT90S8535 0x1e9303
#define DEV_ATMEGA103 0x1e0101
#define DEV_ATMEGA603 0x1e0601*/


#ifdef DEV_INFO
meminfo mem_pic16c84[] =
{
  { 0x0000, 0x07ff, 0x10, 0x00, 0x3fff, AREA_PROGRAM },
  { 0x4200, 0x427f, 0xff, 0x3f, 0x00ff, AREA_DATA },
  { 0x4000, 0x4007, 0x10, 0x00, 0x000f, AREA_ID },
  { 0x400e, 0x400f, 0xf1, 0x3f, 0x001f, AREA_CONFIG }
};

meminfo mem_pic16f84[] =
{
  { 0x0000, 0x07ff, 0x10, 0x00, 0x3fff, AREA_PROGRAM },
  { 0x4200, 0x427f, 0x00, 0x3f, 0x00ff, AREA_DATA },
  { 0x4000, 0x4007, 0x10, 0x00, 0x000f, AREA_ID },
  { 0x400e, 0x400f, 0xf9, 0x3f, 0x3fff, AREA_CONFIG }
};


meminfo mem_at90s1200[] =
{
  { 0x00000, 0x003ff, 0xff, 0xff, 0xffff, AREA_PROGRAM }, /* Not tested */
  { 0x10000, 0x1003f, 0xff, 0xff, 0xffff, AREA_DATA }
};
flaginfo flag_at90s1200[] =
{
  { "", NULL, 0x1e9001, AGRP_DEVID },
  { "WRITE", "Write protect flash & EEPROM",        0xfffdffff, AGRP_LOCK  },
  { "FULL",  "Read & write protect flash & EEPROM", 0xfff9ffff, AGRP_LOCK  },
  { 0 }
};

meminfo mem_at90s2313[] =
{
  { 0x00000, 0x007ff, 0xff, 0xff, 0xffff, AREA_PROGRAM },
  { 0x10000, 0x1007f, 0xff, 0xff, 0xffff, AREA_DATA }
};
flaginfo flag_at90s2313[] =
{
  { "", NULL, 0x1e9101, AGRP_DEVID },
  { "WRITE", "Write protect flash & EEPROM",        0xfffdffff, AGRP_LOCK  },
  { "FULL",  "Read & write protect flash & EEPROM", 0xfff9ffff, AGRP_LOCK  },
  { 0 }
};

meminfo mem_at90s2323[] =
{
  { 0x00000, 0x007ff, 0xff, 0xff, 0xffff, AREA_PROGRAM }, /* Not tested */
  { 0x10000, 0x1007f, 0xff, 0xff, 0xffff, AREA_DATA }
};
flaginfo flag_at90s2323[] =
{
  { "",     NULL,                            0x1e9102,   AGRP_DEVID },
  { "WRITE", "Write protect flash & EEPROM",        0xfffdffff, AGRP_LOCK  },
  { "FULL",  "Read & write protect flash & EEPROM", 0xfff9ffff, AGRP_LOCK  },
  { "RCEN", "Enable internal RC oscillator", 0xfffeffff, AGRP_FUSE  },
  { 0 }
};

meminfo mem_at90s2333[] =
{
  { 0x00000, 0x007ff, 0xff, 0xff, 0xffff, AREA_PROGRAM }, /* Not tested */
  { 0x10000, 0x1007f, 0xff, 0xff, 0xffff, AREA_DATA }
};
flaginfo flag_at90s2333[] =
{
  { "", NULL, 0x1e9105,  AGRP_DEVID },
  { "WRITE", "Write protect flash & EEPROM",        0xfffdffff, AGRP_LOCK  },
  { "FULL",  "Read & write protect flash & EEPROM", 0xfff9ffff, AGRP_LOCK  },
  { "BODEN",    "Enable brown-out detection",                    0xfff7ffff, AGRP_FUSE },
  { "BODLEVEL", "Set brown-out level to 4.0V (instead of 2.7V)", 0xffefffff, AGRP_FUSE },
  { "CKSEL0",   "Select reset delay",                            0xfffeffff, AGRP_FUSE },
  { "CKSEL1",   "Select reset delay",                            0xfffdffff, AGRP_FUSE },
  { "CKSEL2",   "Select reset delay",                            0xfffbffff, AGRP_FUSE },
  { "RD4+6",    "Reset delay = 4 ms  + 6 CK   (= CKSEL2,CKSEL1,CKSEL0)", 0xfff8ffff, AGRP_FUSE },
  { "RD6",      "Reset delay = 6 CK           (= CKSEL2,CKSEL1)",        0xfff9ffff, AGRP_FUSE },
  { "RD64+16K", "Reset delay = 64 ms + 16K CK (= CKSEL2,CKSEL0)",        0xfffaffff, AGRP_FUSE },
  { "RD4+16K",  "Reset delay = 4 ms  + 16K CK (= CKSEL2)",               0xfffbffff, AGRP_FUSE },
  { "RD16K",    "Reset delay = 16K CK         (= CKSEL1,CKSEL0)",        0xfffcffff, AGRP_FUSE },
  { "RD64+1K",  "Reset delay = 64 ms + 1K CK  (= CKSEL1)",               0xfffdffff, AGRP_FUSE },
  { "RD4+1K",   "Reset delay = 4 ms  + 1K CK  (= CKSEL0)",               0xfffeffff, AGRP_FUSE },
  { "RD1K",     "Reset delay = 1K CK          (= )",                     0xffffffff, AGRP_FUSE },
  { 0 }
};

meminfo mem_at90s2343[] =
{
  { 0x00000, 0x007ff, 0xff, 0xff, 0xffff, AREA_PROGRAM }, /* Not tested */
  { 0x10000, 0x1007f, 0xff, 0xff, 0xffff, AREA_DATA }
};
flaginfo flag_at90s2343[] =
{
  { "", NULL, 0x1e9103, AGRP_DEVID },
  { "WRITE", "Write protect flash & EEPROM",        0xfffdffff, AGRP_LOCK  },
  { "FULL",  "Read & write protect flash & EEPROM", 0xfff9ffff, AGRP_LOCK  },
  { 0 }
};

meminfo mem_at90s4414[] =
{
  { 0x00000, 0x00fff, 0xff, 0xff, 0xffff, AREA_PROGRAM }, /* Not tested */
  { 0x10000, 0x100ff, 0xff, 0xff, 0xffff, AREA_DATA }
};
flaginfo flag_at90s4414[] =
{
  { "", NULL, 0x1e9201, AGRP_DEVID },
  { "WRITE", "Write protect flash & EEPROM",        0xfffdffff, AGRP_LOCK  },
  { "FULL",  "Read & write protect flash & EEPROM", 0xfff9ffff, AGRP_LOCK  },
  { 0 }
};

meminfo mem_at90s4433[] =
{
  { 0x00000, 0x00fff, 0xff, 0xff, 0xffff, AREA_PROGRAM }, /* Not tested */
  { 0x10000, 0x100ff, 0xff, 0xff, 0xffff, AREA_DATA }
};
flaginfo flag_at90s4433[] =
{
  { "", NULL, 0x1e9202, AGRP_DEVID },
  { "WRITE", "Write protect flash & EEPROM",        0xfffdffff, AGRP_LOCK  },
  { "FULL",  "Read & write protect flash & EEPROM", 0xfff9ffff, AGRP_LOCK  },
  { "BODEN",    "Enable brown-out detection",                    0xfff7ffff, AGRP_FUSE },
  { "BODLEVEL", "Set brown-out level to 4.0V (instead of 2.7V)", 0xffefffff, AGRP_FUSE },
  { "CKSEL0",   "Select reset delay",                            0xfffeffff, AGRP_FUSE },
  { "CKSEL1",   "Select reset delay",                            0xfffdffff, AGRP_FUSE },
  { "CKSEL2",   "Select reset delay",                            0xfffbffff, AGRP_FUSE },
  { "RD4+6",    "Reset delay = 4 ms  + 6 CK   (= CKSEL2,CKSEL1,CKSEL0)", 0xfff8ffff, AGRP_FUSE },
  { "RD6",      "Reset delay = 6 CK           (= CKSEL2,CKSEL1)",        0xfff9ffff, AGRP_FUSE },
  { "RD64+16K", "Reset delay = 64 ms + 16K CK (= CKSEL2,CKSEL0)",        0xfffaffff, AGRP_FUSE },
  { "RD4+16K",  "Reset delay = 4 ms  + 16K CK (= CKSEL2)",               0xfffbffff, AGRP_FUSE },
  { "RD16K",    "Reset delay = 16K CK         (= CKSEL1,CKSEL0)",        0xfffcffff, AGRP_FUSE },
  { "RD64+1K",  "Reset delay = 64 ms + 1K CK  (= CKSEL1)",               0xfffdffff, AGRP_FUSE },
  { "RD4+1K",   "Reset delay = 4 ms  + 1K CK  (= CKSEL0)",               0xfffeffff, AGRP_FUSE },
  { "RD1K",     "Reset delay = 1K CK          (= )",                     0xffffffff, AGRP_FUSE },
  { 0 }
};

meminfo mem_at90s4434[] =
{
  { 0x00000, 0x00fff, 0xff, 0xff, 0xffff, AREA_PROGRAM }, /* Not tested */
  { 0x10000, 0x100ff, 0xff, 0xff, 0xffff, AREA_DATA }
};
flaginfo flag_at90s4434[] =
{
  { "", NULL, 0x1e9203, AGRP_DEVID },
  { "WRITE", "Write protect flash & EEPROM",        0xfffdffff, AGRP_LOCK  },
  { "FULL",  "Read & write protect flash & EEPROM", 0xfff9ffff, AGRP_LOCK  },
  { "FSTRT", "Enable short start-up time (1.1 ms instead of 16 ms)", 0xfffeffff, AGRP_FUSE },
  { 0 }
};

meminfo mem_at90s8515[] =
{
  { 0x00000, 0x01fff, 0xff, 0xff, 0xffff, AREA_PROGRAM },
  { 0x10000, 0x101ff, 0xff, 0xff, 0xffff, AREA_DATA }
};
flaginfo flag_at90s8515[] =
{
  { "", NULL, 0x1e9301, AGRP_DEVID },
  { "WRITE", "Write protect flash & EEPROM",        0xfffdffff, AGRP_LOCK  },
  { "FULL",  "Read & write protect flash & EEPROM", 0xfff9ffff, AGRP_LOCK  },
  { 0 }
};

meminfo mem_at90s8535[] =
{
  { 0x00000, 0x01fff, 0xff, 0xff, 0xffff, AREA_PROGRAM }, /* Not tested */
  { 0x10000, 0x101ff, 0xff, 0xff, 0xffff, AREA_DATA }
};
flaginfo flag_at90s8535[] =
{
  { "", NULL, 0x1e9303, AGRP_DEVID },
  { "WRITE", "Write protect flash & EEPROM",        0xfffdffff, AGRP_LOCK  },
  { "FULL",  "Read & write protect flash & EEPROM", 0xfff9ffff, AGRP_LOCK  },
  { "FSTRT", "Enable short start-up time (1.1 ms instead of 16 ms)", 0xfffeffff, AGRP_FUSE },
  { 0 }
};

meminfo mem_atmega103[] =
{
  { 0x00000, 0x0ffff, 0xff, 0xff, 0xffff, AREA_PROGRAM }, /* Not tested */
  { 0x10000, 0x10fff, 0xff, 0xff, 0xffff, AREA_DATA } /* Can only handle low 64KB pgm memory */
};
flaginfo flag_atmega103[] =
{
  { "", NULL, 0x1e0101, AGRP_DEVID },
  { "WRITE", "Write protect flash & EEPROM",        0xfffdffff, AGRP_LOCK  },
  { "FULL",  "Read & write protect flash & EEPROM", 0xfff9ffff, AGRP_LOCK  },
  { "EESAVE", "Don't erase EEPROM data when erasing chip", 0xfff7ffff, AGRP_FUSE },
  { "SUT0",   "Select reset delay",                 0xfffeffff, AGRP_FUSE },
  { "SUT1",   "Select reset delay",                 0xfffdffff, AGRP_FUSE },
  { "RD5",    "Reset delay = 5 CK   (= SUT1,SUT0)", 0xfffcffff, AGRP_FUSE },
  { "RD.5",   "Reset delay = 0.5 ms (= SUT1)",      0xfffdffff, AGRP_FUSE },
  { "RD4",    "Reset delay = 4 ms   (= SUT0)",      0xfffeffff, AGRP_FUSE },
  { "RD16",   "Reset delay = 16 ms  (= )",          0xffffffff, AGRP_FUSE },
  { 0 }
};

meminfo mem_atmega161[] =
{
  { 0x00000, 0x03fff, 0xff, 0xff, 0xffff, AREA_PROGRAM }, /* Not tested */
  { 0x10000, 0x101ff, 0xff, 0xff, 0xffff, AREA_DATA }
};
flaginfo flag_atmega161[] =
{
  { "", NULL, 0x1e9401, AGRP_DEVID },
  { "LBWRITE",  "SPI write protect flash & EEPROM",        0xfffffffe, AGRP_LOCK  },
  { "LBFULL",   "SPI read & write protect flash & EEPROM", 0xfffffffc, AGRP_LOCK  },
  { "LB0WRITE", "Boot loader write protect application code",        0xfffdfffb, AGRP_LOCK  },
  { "LB0FULL",  "Boot loader read & write protect application code", 0xfff9fff3, AGRP_LOCK  },
  { "LB1WRITE", "Boot loader write protect boot code",        0xfffdffef, AGRP_LOCK  },
  { "LB1FULL",  "Boot loader read & write protect boot code", 0xfff9ffcf, AGRP_LOCK  },
  { "BOOTRST",  "Reset vector points to boot loader code",       0xffffffbf, AGRP_FUSE },
  { "BODEN",    "Enable brown-out detection",                    0xfffffff7, AGRP_FUSE },
  { "BODLEVEL", "Set brown-out level to 4.0V (instead of 2.7V)", 0xffffffef, AGRP_FUSE },
  { "CKSEL0",   "Select reset delay", 0xfffffffe, AGRP_FUSE },
  { "CKSEL1",   "Select reset delay", 0xfffffffd, AGRP_FUSE },
  { "CKSEL2",   "Select reset delay", 0xfffffffb, AGRP_FUSE },
  { "EC5+6",    "Ext. clock, reset delay = 5 ms  + 6 CK (= CKSEL2,CKSEL1,CKSEL0)", 0xfffffff8, AGRP_FUSE },
  { "EC6",      "Ext. clock, reset delay = 15 us + 6 CK (= CKSEL2,CKSEL1)",        0xfffffff9, AGRP_FUSE },
  { "EX75+16K", "Ext. crystal, reset delay = 75 ms + 16K CK (= CKSEL2,CKSEL0)",    0xfffffffa, AGRP_FUSE },
  { "EX5+16K",  "Ext. crystal, reset delay = 5 ms  + 16K CK (= CKSEL2)",           0xfffffffb, AGRP_FUSE },
  { "EX16K",    "Ext. crystal, reset delay = 15 us + 16K CK (= CKSEL1,CKSEL0)",    0xfffffffc, AGRP_FUSE },
  { "ER75+1K",  "Ext. resonator, reset delay = 75 ms + 1K CK (= CKSEL1)", 0xfffffffd, AGRP_FUSE },
  { "ER5+1K",   "Ext. resonator, reset delay = 5 ms  + 1K CK (= CKSEL0)", 0xfffffffe, AGRP_FUSE },
  { "ER1K",     "Ext. resonator, reset delay = 15 us + 1K CK (= )",       0xffffffff, AGRP_FUSE },
};

meminfo mem_atmega603[] =
{
  { 0x00000, 0x0ffff, 0xff, 0xff, 0xffff, AREA_PROGRAM }, /* Not tested */
  { 0x10000, 0x107ff, 0xff, 0xff, 0xffff, AREA_DATA }
};
flaginfo flag_atmega603[] =
{
  { "", NULL, 0x1e0601, AGRP_DEVID },
  { "WRITE", "Write protect flash & EEPROM",        0xfffdffff, AGRP_LOCK  },
  { "FULL",  "Read & write protect flash & EEPROM", 0xfff9ffff, AGRP_LOCK  },
  { "EESAVE", "Don't erase EEPROM data when erasing chip", 0xfff7ffff, AGRP_FUSE },
  { "SUT0",   "Select reset delay",                 0xfffeffff, AGRP_FUSE },
  { "SUT1",   "Select reset delay",                 0xfffdffff, AGRP_FUSE },
  { "RD5",    "Reset delay = 5 CK   (= SUT1,SUT0)", 0xfffcffff, AGRP_FUSE },
  { "RD.5",   "Reset delay = 0.5 ms (= SUT1)",      0xfffdffff, AGRP_FUSE },
  { "RD4",    "Reset delay = 4 ms   (= SUT0)",      0xfffeffff, AGRP_FUSE },
  { "RD16",   "Reset delay = 16 ms  (= )",          0xffffffff, AGRP_FUSE },
  { 0 }
};

meminfo mem_attiny12[] =
{
  { 0x00000, 0x003ff, 0xff, 0xff, 0xffff, AREA_PROGRAM }, /* Not tested */
  { 0x10000, 0x1003f, 0xff, 0xff, 0xffff, AREA_DATA }
};
flaginfo flag_attiny12[] =
{
  { "",     NULL,                            0x1e9005,   AGRP_DEVID },
  { "WRITE", "Write protect flash & EEPROM",        0xfffffffd, AGRP_LOCK  },
  { "FULL",  "Read & write protect flash & EEPROM", 0xfffffff9, AGRP_LOCK  },
  { "BODEN",    "Enable brown-out detection",                    0xffffff9f, AGRP_FUSE },
  { "BODLEVEL", "Set brown-out level to 2.7V (instead of 1.8V)", 0xffffff5f, AGRP_FUSE },
  { "CKSEL0",   "Select oscillator mode", 0xffffffde, AGRP_FUSE },
  { "CKSEL1",   "Select oscillator mode", 0xffffffdd, AGRP_FUSE },
  { "CKSEL2",   "Select oscillator mode", 0xffffffdb, AGRP_FUSE },
  { "CKSEL3",   "Select oscillator mode", 0xffffffd7, AGRP_FUSE },
  { "EX1K",     "Ext. crystal, reset delay = 1K CK          (= )", 0xffffffdf, AGRP_FUSE },
  { "EX4+1K",   "Ext. crystal, reset delay = 4 ms  + 1K CK  (= CKSEL0)", 0xffffffde, AGRP_FUSE },
  { "EX60+1K",  "Ext. crystal, reset delay = 60 ms + 1K CK  (= CKSEL1)", 0xffffffdd, AGRP_FUSE },
  { "EX16K",    "Ext. crystal, reset delay = 16K CK         (= CKSEL1,CKSEL0)", 0xffffffdc, AGRP_FUSE },
  { "EX4+16K",  "Ext. crystal, reset delay = 4 ms  + 16K CK (= CKSEL2)", 0xffffffdb, AGRP_FUSE },
  { "EX60+16K", "Ext. crystal, reset delay = 60 ms + 16K CK (= CKSEL2,CKSEL0)", 0xffffffda, AGRP_FUSE },
  { "EX60+1K",  "Ext. low freq xtal, rst dly = 60 ms + 1K CK  (= CKSEL2,CKSEL1)", 0xffffffd9, AGRP_FUSE },
  { "EX60+32K", "Ext. low freq xtal, rst dly = 60 ms + 32K CK (= CKSEL2,CKSEL1,CKSEL0)", 0xffffffd8, AGRP_FUSE },
  { "ERC6",     "Ext. RC osc, reset delay = 6 CK         (= CKSEL3)", 0xffffffd7, AGRP_FUSE },
  { "ERC4+6",   "Ext. RC osc, reset delay = 4 ms  + 6 CK (= CKSEL3,CKSEL0)", 0xffffffd6, AGRP_FUSE },
  { "ERC60+6",  "Ext. RC osc, reset delay = 60 ms + 6 CK (= CKSEL3,CKSEL1)", 0xffffffd5, AGRP_FUSE },
  { "IRC6",     "Int. RC osc, reset delay = 6 CK         (= CKSEL3,CKSEL1,CKSEL0)", 0xffffffd4, AGRP_FUSE },
  { "IRC4+6",   "Int. RC osc, reset delay = 4 ms  + 6 CK (= CKSEL3,CKSEL2)", 0xffffffd3, AGRP_FUSE },
  { "IRC60+6",  "Int. RC osc, reset delay = 60 ms + 6 CK (= CKSEL3,CKSEL2,CKSEL0)", 0xffffffd2, AGRP_FUSE },
  { "EC6",      "Ext. clock, reset delay = 6 CK          (= CKSEL3,CKSEL2,CKSEL1)", 0xffffffd1, AGRP_FUSE },
  { "EC4+6",    "Ext. clock, reset delay = 4 ms + 6 CK   (= CKSEL3,CKSEL2,CKSEL1,CKSEL0)", 0xffffffd0, AGRP_FUSE },
  { 0 }
};

meminfo mem_attiny22[] =
{
  { 0x00000, 0x007ff, 0xff, 0xff, 0xffff, AREA_PROGRAM }, /* Not tested */
  { 0x10000, 0x1007f, 0xff, 0xff, 0xffff, AREA_DATA }
};
flaginfo flag_attiny22[] =
{
  { "",     NULL,                            0x1e9106,   AGRP_DEVID },
  { "WRITE", "Write protect flash & EEPROM",        0xfffdffff, AGRP_LOCK  },
  { "FULL",  "Read & write protect flash & EEPROM", 0xfff9ffff, AGRP_LOCK  },
  { "RCEN", "Enable internal RC oscillator", 0xfffeffff, AGRP_FUSE  },
  { 0 }
};


meminfo mem_24c01[] =
{
  { 0x0000, 0x007f, 0x00, 0x00, 0xffff, 0 }
};

meminfo mem_24c02[] =
{
  { 0x0000, 0x00ff, 0x00, 0x00, 0xffff, 0 }
};

meminfo mem_24c04[] =
{
  { 0x0000, 0x01ff, 0x00, 0x00, 0xffff, 0 }
};

meminfo mem_24c08[] =
{
  { 0x0000, 0x03ff, 0x00, 0x00, 0xffff, 0 }
};

meminfo mem_24c16[] =
{
  { 0x0000, 0x07ff, 0x00, 0x00, 0xffff, 0 }
};

meminfo mem_24c32[] =
{
  { 0x0000, 0x0fff, 0x00, 0x00, 0xffff, 0 }
};

meminfo mem_24c65[] =
{
  { 0x0000, 0x1fff, 0x00, 0x00, 0xffff, 0 }
};

devtype dev_types[] =
{
  { "PIC16C84", pic_handler, 4, mem_pic16c84, NULL},
  { "PIC16F84", pic_handler, 4, mem_pic16f84, NULL},
  { "AT90S1200",avr_handler, 2, mem_at90s1200, flag_at90s1200 },
  { "AT90S2313",avr_handler, 2, mem_at90s2313, flag_at90s2313 },
  { "AT90S2323",avr_handler, 2, mem_at90s2323, flag_at90s2323 },
  { "AT90S2333",avr_handler, 2, mem_at90s2333, flag_at90s2333 },
  { "AT90S2343",avr_handler, 2, mem_at90s2343, flag_at90s2343 },
  { "AT90S4414",avr_handler, 2, mem_at90s4414, flag_at90s4414 },
  { "AT90S4433",avr_handler, 2, mem_at90s4433, flag_at90s4433 },
  { "AT90S4434",avr_handler, 2, mem_at90s4434, flag_at90s4434 },
  { "AT90S8515",avr_handler, 2, mem_at90s8515, flag_at90s8515 },
  { "AT90S8535",avr_handler, 2, mem_at90s8535, flag_at90s8535 },
  { "ATmega103",avr_handler, 2, mem_atmega103, flag_atmega103 },
  { "ATmega161",avr_handler, 2, mem_atmega161, flag_atmega161 },
  { "ATmega603",avr_handler, 2, mem_atmega603, flag_atmega603 },
  { "ATtiny12", avr_handler, 2, mem_attiny12,  flag_attiny12  },
  { "ATtiny22", avr_handler, 2, mem_attiny22,  flag_attiny22  },
  { "24C01",    eep_handler, 1, mem_24c01, NULL },
  { "24C02",    eep_handler, 1, mem_24c02, NULL },
  { "24C04",    eep_handler, 1, mem_24c04, NULL },
  { "24C08",    eep_handler, 1, mem_24c08, NULL },
  { "24C16",    eep_handler, 1, mem_24c16, NULL },
  { "24C32",    eep_handler, 1, mem_24c32, NULL },
  { "24C65",    eep_handler, 1, mem_24c65, NULL },
  { NULL,       NULL,        0, NULL, 0 }
};
#endif
extern devtype dev_types[];


#endif
