/*
Picsy v1.07   PIC and AVR programmer
Copyright (C) 1996-2000  Johan Dahlberg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


  Project: PICSY
  File:    Program.c
  Version: 1.07
  Created: 2000-01-28
  Changed: 2000-01-28
  Author:  Johan Dahlberg, johan_d@lysator.liu.se

  Miscellaneous helper functions
*/

#include <stdio.h>
#include <ctype.h>
#include "types.h"
#include "protos.h"


/* Compares two strings case insensitive

   Returns TRUE if the two strings are equal */
BOOL strucmp (char *str, char *stru)
{
  while (toupper(*str) == toupper(*stru)) {
    if (!*stru) return TRUE;
    ++stru;
    ++str;
  }
  return FALSE;
}

/*
  Locate a specific flag by name, group and/or value
  The <flag> has to match all nonzero fields in <template>

  Returns TRUE on match
*/
BOOL flag_compare (flaginfo *flag, flaginfo *template)
{
  if (template->Name) {
    if (!strucmp (flag->Name, template->Name)) return FALSE;
  }
  if (template->Help) {
    if (!strucmp (flag->Help, template->Help)) return FALSE;
  }
  if (template->Value) {
    if (flag->Value != template->Value) return FALSE;
  }
  if (template->Group) {
    if (flag->Group != template->Group) return FALSE;
  }

  return TRUE;
}


/*
  Find first flag in <array> that matches <template>
  
  Returns NULL or pointer to located flag
*/
flaginfo *flag_find(flaginfo *array, flaginfo *template)
{
  if (!array) return NULL;

  for (; array->Name; ++array) {
    /* Does this flag match the template? */
    if (flag_compare(array, template)) return array;
  }

  return NULL;
}


/*
  List names and help for all flags in <array> that matches <template>
  to stream <file>
  
  Returns FALSE if no flags did match the template
*/
BOOL flag_list(FILE *file, flaginfo *array, flaginfo *template)
{
  BOOL found = FALSE;

  if (!array) return FALSE;

  for (; array->Name; ++array) {
    /* Does this flag match the template? */
    if (flag_compare(array, template)) {
      found = TRUE;
      fprintf (file, " %-9s %s\n", array->Name, array->Help);
    }
  }

  return found;
}


/*
  Find first device in <array> that has a flag matching <template>

  Returns NULL or a pointer to located device
*/
devtype *device_find(devtype *array, flaginfo *template)
{
  if (!array) return NULL;

  for (; array->Name; ++array) {
    if (flag_find(array->Flags, template)) return array;
  }

  return NULL;
}
