/*
Picsy v1.05   PIC16C84 programmer
Copyright (C) 1996-99  Johan Dahlberg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


  Project: PICSY
  File:    Pic.h
  Version: 1.05
  Created: 1996-11-01
  Changed: 1999-02-01
  Author:  Johan Dahlberg, johan_d@lysator.liu.se

  PIC command definitions and data buffers
*/

#include "types.h"

#ifndef PIC_H
#define PIC_H 1

/* PIC programming commands */
#define PCMD_LOADCFG	0x00	/* Load configuration */
#define PCMD_LOADPGM	0x02	/* Load data for program memory */
#define PCMD_READPGM	0x04	/* Read data from program memory */
#define PCMD_INCADDR	0x06	/* Increment address counter */
#define PCMD_PROGRAM	0x08	/* Begin programming */
#define PCMD_LOADDATA	0x03	/* Load data for data memory */
#define PCMD_READDATA	0x05	/* Read data from data memory */
#define PCMD_ERASE1     0x01    /* Bulk erase command 1 */
#define PCMD_ERASE2     0x07    /* Bulk erase command 2 */

/* Data buffer */
#define BUFFER_SIZE	0x20000
#define BUF_EMPTYEVEN	0xff	/* Not-programmed byte-state */
#define BUF_EMPTYODD	0x3f
extern UBYTE buffer[BUFFER_SIZE];	/* Buffer for program data */
extern UBYTE buf_flag[BUFFER_SIZE/16];  /* buf_flag[i] is set if any byte in
					   buffer[i .. i+15] is set on hex file read */


/* Area definitions for program(), verify() and read() in Program.c */
#define AREA_PROGRAM	0x01	/* Program/verify program memory */
#define AREA_DATA	0x02	/* Program/verify data memory */
#define AREA_CONFIG	0x04	/* Program/verify configuration word */
#define AREA_ID         0x08    /* ID bits */



/* AVR programming commands */
#define ACMD_PGMEN      0xac530000 /* Enable serial programming */
#define ACMD_ERASE      0xac800000 /* Chip erase */
#define ACMD_RDPMEM     0x20000000 /* Read program memory */
#define ACMD_WRPMEM     0x40000000 /* Write program memory */
#define ACMD_RDEMEM     0xa0000000 /* Read EEPROM memory */
#define ACMD_WREMEM     0xc0000000 /* Write EEPROM memory */
#define ACMD_RDLOCK     0x58000000 /* Read lock and fuse bits (2323,2343,4434,8535,MEGA) */
#define ACMD_WRLOCK     0xace00000 /* Write lock bits */
#define ACMD_WRRCEN     0xaca00000 /* Write on-chip oscillator register (2323,2343) */
#define ACMD_WRRFSTRT   0xaca00000 /* Write startup-timer bit (4434,8535) */
#define ACMD_RDFUSE     0x50000000 /* Read fuse bits (MEGA) */
#define ACMD_WRFUSE     0xaca00000 /* Write fuse bits (MEGA) */
#define ACMD_RDDEV      0x30000000 /* Read device code */

/* AVR device signature codes */
#define DEV_AT90S1200 0x1e9001
#define DEV_AT90S2313 0x1e9101
#define DEV_AT90S2323 0x1e9704
#define DEV_AT90S2343 0x1e9703
#define DEV_AT90S4414 0x1e9201
#define DEV_AT90S4434 0x1e9203
#define DEV_AT90S8515 0x1e9301
#define DEV_AT90S8535 0x1e9303
#define DEV_ATMEGA103 0x1e0101
#define DEV_ATMEGA603 0x1e0601


#ifdef DEV_INFO
meminfo mem_pic16c84[] =
{
  { 0x0000, 0x07ff, 0x10, 0x00, 0x3fff, AREA_PROGRAM },
  { 0x4200, 0x427f, 0xff, 0x3f, 0x00ff, AREA_DATA },
  { 0x4000, 0x4007, 0x10, 0x00, 0x000f, AREA_ID },
  { 0x400e, 0x400f, 0xf1, 0x3f, 0x001f, AREA_CONFIG }
};

meminfo mem_pic16f84[] =
{
  { 0x0000, 0x07ff, 0x10, 0x00, 0x3fff, AREA_PROGRAM },
  { 0x4200, 0x427f, 0x00, 0x3f, 0x00ff, AREA_DATA },
  { 0x4000, 0x4007, 0x10, 0x00, 0x000f, AREA_ID },
  { 0x400e, 0x400f, 0xf9, 0x3f, 0x3fff, AREA_CONFIG }
};


meminfo mem_at90s1200[] =
{
  { 0x00000, 0x003ff, 0xff, 0xff, 0xffff, AREA_PROGRAM }, /* Not tested */
  { 0x10000, 0x1003f, 0xff, 0xff, 0xffff, AREA_DATA }
};

meminfo mem_at90s2313[] =
{
  { 0x00000, 0x007ff, 0xff, 0xff, 0xffff, AREA_PROGRAM }, /* Not tested */
  { 0x10000, 0x1007f, 0xff, 0xff, 0xffff, AREA_DATA }
};

meminfo mem_at90s2323[] =
{
  { 0x00000, 0x007ff, 0xff, 0xff, 0xffff, AREA_PROGRAM }, /* Not tested */
  { 0x10000, 0x1007f, 0xff, 0xff, 0xffff, AREA_DATA }
};

meminfo mem_at90s2343[] =
{
  { 0x00000, 0x007ff, 0xff, 0xff, 0xffff, AREA_PROGRAM }, /* Not tested */
  { 0x10000, 0x1007f, 0xff, 0xff, 0xffff, AREA_DATA }
};

meminfo mem_at90s4414[] =
{
  { 0x00000, 0x00fff, 0xff, 0xff, 0xffff, AREA_PROGRAM }, /* Not tested */
  { 0x10000, 0x100ff, 0xff, 0xff, 0xffff, AREA_DATA }
};

meminfo mem_at90s4434[] =
{
  { 0x00000, 0x00fff, 0xff, 0xff, 0xffff, AREA_PROGRAM }, /* Not tested */
  { 0x10000, 0x100ff, 0xff, 0xff, 0xffff, AREA_DATA }
};

meminfo mem_at90s8515[] =
{
  { 0x00000, 0x01fff, 0xff, 0xff, 0xffff, AREA_PROGRAM },
  { 0x10000, 0x101ff, 0xff, 0xff, 0xffff, AREA_DATA }
};

meminfo mem_at90s8535[] =
{
  { 0x00000, 0x01fff, 0xff, 0xff, 0xffff, AREA_PROGRAM }, /* Not tested */
  { 0x10000, 0x101ff, 0xff, 0xff, 0xffff, AREA_DATA }
};

meminfo mem_atmega103[] =
{
  { 0x00000, 0x0ffff, 0xff, 0xff, 0xffff, AREA_PROGRAM }, /* Not tested */
  { 0x10000, 0x10fff, 0xff, 0xff, 0xffff, AREA_DATA } /* Can only handle low 64KB pgm memory */
};

meminfo mem_atmega603[] =
{
  { 0x00000, 0x0ffff, 0xff, 0xff, 0xffff, AREA_PROGRAM }, /* Not tested */
  { 0x10000, 0x107ff, 0xff, 0xff, 0xffff, AREA_DATA }
};

meminfo mem_24c01[] =
{
  { 0x0000, 0x007f, 0x00, 0x00, 0xffff, 0 }
};

meminfo mem_24c02[] =
{
  { 0x0000, 0x00ff, 0x00, 0x00, 0xffff, 0 }
};

meminfo mem_24c04[] =
{
  { 0x0000, 0x01ff, 0x00, 0x00, 0xffff, 0 }
};

meminfo mem_24c08[] =
{
  { 0x0000, 0x03ff, 0x00, 0x00, 0xffff, 0 }
};

meminfo mem_24c16[] =
{
  { 0x0000, 0x07ff, 0x00, 0x00, 0xffff, 0 }
};

meminfo mem_24c32[] =
{
  { 0x0000, 0x0fff, 0x00, 0x00, 0xffff, 0 }
};

meminfo mem_24c65[] =
{
  { 0x0000, 0x1fff, 0x00, 0x00, 0xffff, 0 }
};

devtype dev_types[] =
{
  { "PIC16C84", pic_handler, 4, mem_pic16c84, 0},
  { "PIC16F84", pic_handler, 4, mem_pic16f84, 0},
  { "AT90S1200",avr_handler, 2, mem_at90s1200, DEV_AT90S1200 },
  { "AT90S2313",avr_handler, 2, mem_at90s2313, DEV_AT90S2313 },
  { "AT90S2323",avr_handler, 2, mem_at90s2323, DEV_AT90S2323 },
  { "AT90S2343",avr_handler, 2, mem_at90s2343, DEV_AT90S2343 },
  { "AT90S4414",avr_handler, 2, mem_at90s4414, DEV_AT90S4414 },
  { "AT90S4434",avr_handler, 2, mem_at90s4434, DEV_AT90S4434 },
  { "AT90S8515",avr_handler, 2, mem_at90s8515, DEV_AT90S8515 },
  { "AT90S8535",avr_handler, 2, mem_at90s8535, DEV_AT90S8535 },
  { "ATMEGA103",avr_handler, 2, mem_atmega103, DEV_ATMEGA103 },
  { "ATMEGA603",avr_handler, 2, mem_atmega603, DEV_ATMEGA603 },
  { "24C01",    eep_handler, 1, mem_24c01, 0 },
  { "24C02",    eep_handler, 1, mem_24c02, 0 },
  { "24C04",    eep_handler, 1, mem_24c04, 0 },
  { "24C08",    eep_handler, 1, mem_24c08, 0 },
  { "24C16",    eep_handler, 1, mem_24c16, 0 },
  { "24C32",    eep_handler, 1, mem_24c32, 0 },
  { "24C65",    eep_handler, 1, mem_24c65, 0 },
  { NULL,       NULL,        0, NULL, 0 }
};
#endif
extern devtype dev_types[];


#endif
