/*
Picsy v1.07   PIC16C84 programmer
Copyright (C) 1996-2000  Johan Dahlberg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


  Project: PICSY
  File:    Protos.h
  Version: 1.07
  Created: 1996-10-25
  Changed: 2000-01-28
  Author:  Johan Dahlberg, johan_d@lysator.liu.se

  Prototypes for all functions
*/

#include <stdio.h>
#include "types.h"

#ifndef PROTOS_H
#define PROTS_H 1

/* Picsy.c */
int pic_handler (UBYTE cmd, char *filename, devtype *device);
int avr_handler (UBYTE cmd, char *filename, devtype *device);
int eep_handler (UBYTE cmd, char *filename, devtype *device);

/* Misc.c */
BOOL      strucmp (char *str, char *stru);
BOOL      flag_compare (flaginfo *flag, flaginfo *template);
flaginfo *flag_find(flaginfo *array, flaginfo *template);
BOOL      flag_list(FILE *file, flaginfo *array, flaginfo *template);
devtype  *device_find(devtype *array, flaginfo *template);

/* Hex.c */
void clearbuf (devtype *device);
BOOL readhex (char *filename, devtype *device, BOOL extended);
BOOL writehex(char *filename, devtype *device, BOOL extended);
void dump(void);

/* Program.c */
BOOL enablevpp(void);
BOOL testmode(void);
BOOL pic_program(devtype *device);
BOOL pic_verify(devtype *device);
BOOL pic_read(devtype *device);
BOOL pic_erase(devtype *device);
void sendcmd(UBYTE cmd);
void writedata(UWORD data);
UWORD readdata(void);

BOOL avr_program(devtype *device);
BOOL avr_verify(devtype *device);
BOOL avr_read(devtype *device);
BOOL avr_erase(devtype *device);
BOOL avr_lock(char *mode, devtype *device);
BOOL avr_fuse(char *mode, devtype *device);

BOOL eep_program(devtype *device);
BOOL eep_verify(devtype *device);
BOOL eep_read(devtype *device);
BOOL eep_erase(devtype *device);


/* Hardware.c */
void hw_info(void);
void hw_usage(void);
BOOL hw_init(char *string);
void hw_cleanup(void);
void hw_vppon(void);
void hw_vppoff(void);
void hw_waitus(void);
void hw_waitms(UBYTE n);
void hw_dataout(BOOL level);
void hw_clock(BOOL level);
BOOL hw_datain(void);


#ifdef DEBUG
#define PDEBUG(args) printf args
#else
#define PDEBUG(args)
#endif

#endif
