
/*
Picsy v1.07   PIC16C84 programmer
Copyright (C) 1996-2000  Johan Dahlberg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


  Project: PICSY
  File:    Types.h
  Version: 1.07
  Created: 1996-10-25
  Changed: 2000-01-27
  Author:  Johan Dahlberg, johan_d@lysator.liu.se

  C type definitions
*/


#ifndef TYPES_H
#define TYPES_H 1

typedef signed   long  LONG;		/* Signed types */
typedef signed   short WORD;
typedef signed   char  BYTE;
typedef unsigned long  ULONG;		/* Unsigned types */
typedef unsigned short UWORD;
typedef unsigned char  UBYTE;
typedef int            BOOL;		/* Boolean: TRUE or FALSE */

#define TRUE  1
#define FALSE 0


/* Memory block definition */
typedef struct
{
  ULONG FirstAddr;    /* First address in block */
  ULONG LastAddr;     /* Last address in block */
  UBYTE DefaultEven;  /* Default data on even address */
  UBYTE DefaultOdd;   /* Default data on odd address */
  UWORD Mask;         /* Mask for data read from the device */
  ULONG UserData;     /* Data depending on device handler */
} meminfo;

/* Device flag definition */
typedef struct
{
  char  *Name;  /* Symbolic flag name, NULL if end of flaginfo array */
  char  *Help;  /* Description */
  ULONG  Value; /* Flag value */
  ULONG  Group; /* Flag group number (or whatever this field will be used for) */
} flaginfo;


/* Device type */
typedef struct
{
  char     *Name;    /* Device type name */
  int (*Handler)(UBYTE, char*, void*); /* Device programmer handler */
  int      Blocks;   /* Number of memory blocks */
  meminfo  *MemInfo; /* Memory block information */
  flaginfo *Flags;   /* Device flags (depending on  device handler) */
} devtype;

#endif
